// Syntex of iTunes Search API
String json_url = "https://itunes.apple.com/search?term=billie+eilish&limit=50";
/* Explanation about the search result 
https://developer.apple.com/library/archive/documentation/AudioVideo/Conceptual/iTuneSearchAPI/UnderstandingSearchResults.html
*/
/* How to construct a API search
https://developer.apple.com/library/archive/documentation/AudioVideo/Conceptual/iTuneSearchAPI/Searching.html#//apple_ref/doc/uid/TP40017632-CH5-SW1
*/
// Initialize variables
PImage[] covers;
String[] titles;
String[] releaseDates;
String[] kinds;
String[] collectionCensoredNames;
String[] countries;
int rows;
int cols = 10;
int index = 0;
int resultCounts =0;
int indexText = 0;
float edge = 4;

void setup() {

  size(1300, 800);
  
  // Conduct iTunes search
  JSONObject billie_eilish = loadJSONObject(json_url); 
  // Save the searching result
  saveJSONObject(billie_eilish, "data/billieEilish.json");
  // Save the length of the searching results
  resultCounts = billie_eilish.getInt("resultCount");
  // Initialize variables to save information in the JSON file
  covers = new PImage[resultCounts];
  titles = new String[resultCounts];
  releaseDates = new String[resultCounts];
  kinds = new String[resultCounts];
  countries = new String[resultCounts];
  collectionCensoredNames = new String[resultCounts];
  
  // Compute how many rows are needed
  rows = int(resultCounts/cols)+1;
  // Load the result
  JSONArray items = billie_eilish.getJSONArray("results");
  println(items.size());

  for (int i =0; i<items.size(); i++) 
  {
    // Obtain the image URL
    String imageURL = items.getJSONObject(i).getString("artworkUrl100");
    /* Load the information in the url. We fetched keywords including 
    image, trackName, releaseDate and kind*/
    covers[i] = loadImage(imageURL);
    titles[i] = items.getJSONObject(i).getString("trackName");
    releaseDates[i] = items.getJSONObject(i).getString("releaseDate").substring(0, 10);
    kinds[i] = items.getJSONObject(i).getString("kind");
    collectionCensoredNames[i] = items.getJSONObject(i).getString("collectionCensoredName");
    countries[i] = items.getJSONObject(i).getString("country");
    println(imageURL);
  }
}

void draw() {
  background(200);
  /* If drawing the texts in the same loop of drawing the images, 
  the text will be covered by the next image*/
  // First draw the image
  index = 0;
  for (int i =0; i<rows; i++) {
    for (int j =0; j<cols; j++) {
      if(index<resultCounts){
        float x_pos = 150+100*j;
        float y_pos = 100*i;
        // Display the image on the canvas
        image(covers[index], x_pos, y_pos);
        index +=1;
      } 
    }
  }

  // Second, set the mouse rolling over effect
  indexText = 0;
  for (int i =0; i<rows; i++) {
    for (int j =0; j<cols; j++) {
      if(indexText<resultCounts){
        // Compute the center of the image
        float x_pos = 100*j+150+50;
        float y_pos = 100*i+50;   
        // calculate the distance between (p_x, p_y) , (mouseX, mouseY)
        float distanceObjectMouse = sq(x_pos - mouseX) + sq(y_pos-mouseY);
        // draw title if the mouse is near the image 
        if (distanceObjectMouse < 1000) {

          pushMatrix();
          // draw the text close to the image's center 
          translate(x_pos+10, y_pos+10);
          // set the background of the text
          fill(#FFFFED,230);
          noStroke();

          String str = titles[indexText]+", " +kinds[indexText]+ ", released on "+releaseDates[indexText]+
        ", "+countries[indexText] + ", "+collectionCensoredNames[indexText];
          PFont f = createFont("Arial", 14);
          textFont(f);
          textLeading(20); 
          textAlign(LEFT);

          rect(0, 0, 155+2*edge, 2*edge-20+(20+f.getSize())*textWidth(str)/155);
          
          fill(0);
          text(str, edge,edge, 155, -20+(20+f.getSize())*textWidth(str)/155);
          
          popMatrix();
        }
        indexText+=1;
        
      }
    }
  }
}
